<?php 

namespace Management\Controllers;

use Management\Services\ManageDashboardService;
use Management\Services\SetFilialSelecionada;
use Zeedhi\Framework\DTO;
use Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\Util\Functions;

class ManageDashboardController {    

    protected $ManageDashboardService, 
              $filialSelecionadaService;

    public function __construct (ManageDashboardService $ManageDashboardService, SetFilialSelecionada $filialSelecionadaService, $environment) {
        $this->ManageDashboardService = $ManageDashboardService;
        $this->filialSelecionadaService = $filialSelecionadaService;
        $this->environment = $environment;
    }
    
    private function returnRequestFilter($request) {
        $arr = array();
        foreach ($request->getFilterCriteria()->getConditions() as $condition) {
            $arr[ $condition['columnName'] ] = $condition['value'];  
        }
        return $arr;
    }
    
    public function getVendasMensais(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $this->returnRequestFilter($request);
        
        $NRORG = $conditions['NRORG'];
        $CDFILIAL = $conditions['CDFILIAL'];
        
        $retornoTurnos = $this->ManageDashboardService->getVendasMensais($CDFILIAL,$NRORG);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVendasMensais', $retornoTurnos));
    
    }
    
    public function findObjetivoVendaDiaAlt(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->returnRequestFilter($request);
        
        $NRORG         = $conditions['NRORG'];
        $CDFILIAL      = $conditions['CDFILIAL'];
        $CDOBJANO      = $conditions['CDOBJANO'];
        $NRDIASEMANA   = $conditions['NRDIASEMANA'];
        $DTALT         = $conditions['DTALT'];
        $CDOBJMES      = $conditions['CDOBJMES'];
        
        $retornoTurnos = $this->ManageDashboardService->getObjetivoVendaDiaAlt($CDFILIAL,$NRORG,$CDOBJANO,$NRDIASEMANA,$DTALT,$CDOBJMES);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('objetivoVendaDiaAlt', $retornoTurnos));
    
    }
    
    public function findObjetivoVendaDia(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->returnRequestFilter($request);
        
        $NRORG          = $conditions['NRORG'];
        $CDFILIAL       = $conditions['CDFILIAL'];
        $CDOBJANO       = $conditions['CDOBJANO'];
        $CDOBJDIA       = $conditions['CDOBJDIA'];
        $CDOBJMES       = $conditions['CDOBJMES'];
        
        $retornoTurnos = $this->ManageDashboardService->getObjetivoVendaDia($CDFILIAL,$NRORG,$CDOBJANO,$CDOBJDIA,$CDOBJMES);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('objetivoVendaDia', $retornoTurnos));
    
    }
    
    public function findObjetivoVendaMes(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->returnRequestFilter($request);
        
        $NRORG      =   $conditions['NRORG'];
        $CDFILIAL   =   $conditions['CDFILIAL'];
        $CDOBJANO   =   $conditions['CDOBJANO'];
        $CDOBJMES   =   $conditions['CDOBJMES'];
        
        $retornoTurnos = $this->ManageDashboardService->getObjetivoVendaMes($CDFILIAL,$NRORG,$CDOBJANO,$CDOBJMES);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('objetivoVendaMes', $retornoTurnos));
    
    }
    
    public function getVendasDiarias(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->returnRequestFilter($request);
        
        $NRORG        =   $conditions['NRORG'];
        $CDFILIAL     =   $conditions['CDFILIAL'];
        $DTENTRVENDA  =   $conditions['DTENTRVENDA'];
        
        $retornoTurnos = $this->ManageDashboardService->getVendasDiarias($CDFILIAL,$DTENTRVENDA,$NRORG);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVendasDiarias', $retornoTurnos));
    }
    
    public function getProdutosMaisVendidos(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->returnRequestFilter($request);
        
        $NRORG          =   $conditions['NRORG'];
        $CDFILIAL       =   $conditions['CDFILIAL'];
        $DTENTRVENDA    =   $conditions['DTENTRVENDA'];
        
        $retornoTurnos = $this->ManageDashboardService->getProdutosMaisVendidos($CDFILIAL,$DTENTRVENDA,$NRORG);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getProdutosMaisVendidos', $retornoTurnos));
    }
    
    public function findDivergenciasConciliacao (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        
        $NRORG  =   $condition['NRORG'];
        
        $retornoTurnos = $this->ManageDashboardService->findDivergenciasConciliacao($NRORG);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('findDivergenciasConciliacao', $retornoTurnos));
    }
    
    public function setFilialSelecionadaOperador (DTO\Request\Row $request, DTO\Response $response) {
        $condition = $request->getRow();
        
        $CDOPERADOR =   $condition['CDOPERADOR'];
        $NRORG      =   $condition['NRORG'];
            
        $filiais = $this->ManageDashboardService->findFiliaisDoOperador($CDOPERADOR, $NRORG);
        
        $result = array();
        
        if((is_array($filiais) ? count($filiais) : 0) === 1) {
            $CDFILIAL =  $filiais[0]['CDFILIAL'];
            $NMFILIAL =  $filiais[0]['NMFILIAL'];
            
            $condition['CDFILIAL'] = $CDFILIAL;
            
            $this->filialSelecionadaService->changeUserUnit($condition);
            
            $result['IDREDIRECT'] = false;
            $result['CDFILIALSELECIONADA'] = $CDFILIAL;
            $result['NMFILIAL'] = $NMFILIAL;
        } else {
            $result['IDREDIRECT'] = true;
        }
        
        $response->addDataSet( new \Zeedhi\Framework\DataSource\DataSet('filialSelecionada', $result ));
    }
    
    public function findConsolidacaoVendaPendente (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        
        $CDOPERADOR  =   $condition['CDOPERADOR'];
        $NRORG       =   $condition['NRORG'];
        
        if(Functions::arrayKeyExists('CDFILIAL', $condition)) {
            $CDFILIAL =   $condition['CDFILIAL'];
        } else {
            $CDFILIAL = false;
        }
        
        $retornoTurnos = $this->ManageDashboardService->findConsolidacaoVendaPendente($NRORG, $CDOPERADOR, $CDFILIAL);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getConsolidacaoVendaPendente', $retornoTurnos));
    }
    
    public function findFechamentoCupomPendente (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        
        $NRORG  =   $condition['NRORG'];
        $CDOPERADOR  =   $condition['CDOPERADOR'];
        
        if(Functions::arrayKeyExists('CDFILIAL', $condition)) {
            $CDFILIAL =   $condition['CDFILIAL'];
        } else {
            $CDFILIAL = false;
        }
        
        $retornoTurnos = $this->ManageDashboardService->findFechamentoCupomPendente($NRORG, $CDOPERADOR, $CDFILIAL);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getFechamentoCupomPendente', $retornoTurnos));
    }
    
    public function findFechamentoCaixaPendente (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        
        $NRORG  =   $condition['NRORG'];
        $CDOPERADOR  =   $condition['CDOPERADOR'];
        
        if(Functions::arrayKeyExists('CDFILIAL', $condition)) {
            $CDFILIAL =   $condition['CDFILIAL'];
        } else {
            $CDFILIAL = false;
        }
        
        $retornoTurnos = $this->ManageDashboardService->findFechamentoCaixaPendente($NRORG, $CDOPERADOR, $CDFILIAL);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getFechamentoCaixaPendente', $retornoTurnos));
    }
    
    public function checkFilialDisp (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        $unidadeBloq = $this->ManageDashboardService->unidadeBloq($condition);
        if($unidadeBloq){
            $retorno = $this->ManageDashboardService->perfilUtlBloq($condition);
        } else {
            $retorno = true;
        }
        $UnidadeDisponivel = array($retorno);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('checkFilialDisp', $UnidadeDisponivel));
    }
    
    public function getParamOperador (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        $paramOperador = $this->ManageDashboardService->paramOperador($condition);
        if((is_array($paramOperador) ? count($paramOperador) : 0) == 0 || $paramOperador == false){
            $paramOperador = array(
                'CDPERFILACESSO' => '',
        	    'NMPERFILACESSO' => '',
        	    'CDOPERADOR' => null);
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('paramOperador', $paramOperador));
    }
    
    public function getParametersAccessControl (DTO\Request\Filter $request, DTO\Response $response) {
        $NRORG = $this->environment->getNrOrg();
        $CDPERFILSELECIONADO =  $this->ManageDashboardService->paramPerfilSelecionado($this->environment->getNrOrg(), $this->environment->getCdOperador());
        $return = array(
            'IDUTLPERFILFILIAL'   => $this->ManageDashboardService->getIdUtlPerfilFilial($NRORG),
            'CDPERFILSELECIONADO' => $CDPERFILSELECIONADO['CDPERFILSELECIONADO']);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getParametersAccessControl', $return));
    }
    
    /* Dashboard 3 */
    
    public function findV_QTTICKET (DTO\Request\Filter $request, DTO\Response $response) {
        $condition = $this->returnRequestFilter($request);
        $this->setTCDandCDcondition($condition, 'CDFILIAL', 'CDFILIAL');
        
        /*
        
        $NRORG       =   $condition['NRORG'];
        $CDFILIAL    =   $condition['CDFILIAL'];
        $DTVNDCARDS  =   $condition['DTVNDCARDS'];
        $CDOPERADOR  =   $condition['CDOPERADOR'];
        $CHECK  =   isset($condition['CHECK']) ? $condition['CHECK'] : 'S';
        $TCDFILIAL  =   $condition['TCDFILIAL'];
        
        $retornoCards = $this->ManageDashboardService->findV_QTTICKET($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoGraph = $this->ManageDashboardService->findSaleGraph($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoSaleTypeGraph = $this->ManageDashboardService->findSaleTypeGraph($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoSalePaymentGraph = $this->ManageDashboardService->findSalePaymentGraph($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoProdutoMaiorCusto = $this->ManageDashboardService->findProdutoMaiorCusto($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoProdutoMenorCusto = $this->ManageDashboardService->findProdutoMenorCusto($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoProdMaisAjustes = $this->ManageDashboardService->findProdMaisAjustes($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoContasPagar = $this->ManageDashboardService->findContasPagar($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoContasReceber = $this->ManageDashboardService->findContasReceber($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoSaldoEmContas = $this->ManageDashboardService->findSaldoEmContas($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoStatusNota = $this->ManageDashboardService->findStatusNotaTab($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoSemContagem = $TCDFILIAL == 'T' ? $this->ManageDashboardService->findUnidadeSemContagem($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL) : $this->ManageDashboardService->findUltimaContagem($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $retornoSaleForDayGraph = $this->ManageDashboardService->findSaleForDayGraph($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getCards', $retornoCards));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getGraph', $retornoGraph));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getSaleTypeGraph', $retornoSaleTypeGraph));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getSalePaymentGraph', $retornoSalePaymentGraph));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getSaleForDayGraph', $retornoSaleForDayGraph));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_MAIORCUSTO', $retornoProdutoMaiorCusto));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_MENORCUSTO', $retornoProdutoMenorCusto));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_PRODMAISAJUST', $retornoProdMaisAjustes));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_CONTAGEMPEND', $retornoSemContagem));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_CONTASPAGAR', $retornoContasPagar));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_CONTASRECEBER', $retornoContasReceber));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_SALDOEMCONTAS', $retornoSaldoEmContas));
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_STATUSNOTA', $retornoStatusNota));
        
        $retornoCMVTab = $this->ManageDashboardService->findCMVTab($NRORG, $CDFILIAL, $DTVNDCARDS, $CDOPERADOR, $CHECK, $TCDFILIAL);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getT_CMV', $retornoCMVTab));   */
    }
    
    private function setConditions($request){
        $condition = $this->returnRequestFilter($request);
        
        $condition['NRORG'] = $this->environment->getNrOrg();
        $condition['CDOPERADOR'] = $this->environment->getCdOperador();
        $condition['CDFILIALLOG'] = $this->environment->getCdFilial();
        $condition['CDOPERADORLOG'] = $this->environment->getCdOperador();
        $condition['PERFACESSFILI'] = $this->environment->getIdUtlPerfilFilial();
        
        if($this->environment->getIdUtlPerfilFilial() == 'S'){
            $CDPERFILSELECIONADO =  $this->ManageDashboardService->paramPerfilSelecionado($this->environment->getNrOrg(), $this->environment->getCdOperador());    
            $condition['CDPERFILSELECIONADO'] = $CDPERFILSELECIONADO['CDPERFILSELECIONADO'];
        }else{
            $condition['CDPERFILSELECIONADO'] = null;
        }
        
        if(empty($condition['CDFILIAL']) && empty($condition['FILTERDASH'])){
            $condition['CDFILIAL'] = $this->environment->getCdFilial();
            $condition['CDFILIALP'] = '';
            $condition['TCDFILIALP'] = 'T';
        }else{
            $condition['CDFILIALP'] = '';
            $condition['TCDFILIALP'] = 'T';
        }
        
        //se a filial filtrada for a mesma da logada mantem trazer todas as unidades
        if(!is_array($condition['CDFILIAL'])){
            if($condition['CDFILIAL'] == $condition['CDFILIALLOG']){
                $condition['CDFILIALP'] = '';
                $condition['TCDFILIALP'] = 'T';
            }else{
                $condition['CDFILIALP'] = $condition['CDFILIAL'];
                $condition['TCDFILIALP'] = '';
            }
        }else{
            
            if(empty($condition['CDFILIAL'])){
                $condition['CDFILIALP'] = $condition['CDFILIAL'];
                $condition['TCDFILIALP'] = 'T';
            }else{
                if(count($condition['CDFILIAL']) > 1){
                    $condition['CDFILIALP'] = $condition['CDFILIAL'];
                    $condition['TCDFILIALP'] = '';    
                }else{
                   if($condition['CDFILIAL'][0] == $condition['CDFILIALLOG']){
                        $condition['CDFILIALP'] = '';
                        $condition['TCDFILIALP'] = 'T';
                   }else{
                        $condition['CDFILIALP'] = $condition['CDFILIAL'][0];
                        $condition['TCDFILIALP'] = '';
                   } 
                }
            }
        }
        return $condition;
    }
    
    public function findSetStripe(DTO\Request\Filter $request, DTO\Response $response){
        
        $condition = $this->setConditions($request);
        $getFilter = [];
        $getFilter['DATA'] = $condition['DTVNDCARDS'];
        
        if(empty($condition['CDFILIAL'])){
            $getFilter['FILIAL'] = 'Todas as unidades filtradas';
        }else{
            if(is_array($condition['CDFILIAL'])){
                if(count($condition['CDFILIAL']) > 1){
                    $getFilter['FILIAL'] = 'Lista com '. count($condition['CDFILIAL']).' unidades selecionadas.';
                }else{
                    $getFilter['FILIAL'] = $this->ManageDashboardService->findGetFilial($condition['CDFILIAL'][0]);
                }
            }else{
                $getFilter['FILIAL'] = $this->ManageDashboardService->findGetFilial($condition['CDFILIAL']);
            }
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getFilter', $getFilter));   
    }
    
    public function findSetHelp(DTO\Request\Filter $request, DTO\Response $response){
        
        //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        $getInfo['type'] = $type;
        $getInfo['informacoes'] = '';
        $mensagem = '';
        
        if($type == 'P'){
            
            $mensagem .= 'Dashboard (Painel de Controle) Padrão';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            $mensagem .= '-----------------------------------------------------CARDS------------------------------------------------------------';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= 'Todos os cards acompanham o filtro da Tela';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= 'VENDAS CONCLUÍDAS: Quantidade de vendas consolidads com o status de concluídas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDAS CANCELADAS: Quantidade de vendas consolidads com o status de canceladas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'FECH. DE CUPONS PENDENTES: Quantidade de cupons pendentes dos últimos 30 dias de acordo com data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'ESTOQUE NÃO CONSOLIDADOS: Quantidade de itens/produtos não consolidados das vendas concluídas dos últimos 30 dias de acordo com data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'FECH. DE CAIXAS PENDENTES: Quantidade de caixas com fechamento pendente dos últimos 30 dias de acordo com data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'INCONSISTÊNCIAS DE INTEGRAÇÃO: Quantidade de erros no Logs! Obs.: Card exibido somente quando usado integração do Conciliador';
            $mensagem .= PHP_EOL;
            
            $mensagem .= PHP_EOL;
            
            $mensagem .= '-----------------------------------------------------GRAFICOS------------------------------------------------------------';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= 'Os gráficos consideram as Unidades/Filiais de acesso do Operador logado por padrão, salvo utilização do filtro de Unidade diferente da logada!';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= 'VENDAS REALIZADAS/DIA: As 10 unidades com maior valor de vendas consolidadas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDAS REALIZADAS/MODALIDADE POR DIA: Total por modalidade das vendas consolidadas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDAS ACUMULADAS/DIA: Valor de venda por hora da Unidade dos valores consolidados!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDAS X META ACUMULADA: Porcentagem da venda por valor da Meta Acumulada da Unidade!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TICKET MÉDIO - MÉDIA/DIA: As 10 unidades com maior ticket médio de vendas consolidadas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDA POR UNIDADE/DIA (PORCENTAGEM): Porcentagem do valor vendido por unidade!';
            $mensagem .= PHP_EOL;
           
            
        }else if($type == 'C'){
            
            $mensagem .= 'Dashboard (Painel de Controle) Corporativo';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            $mensagem .= '-----------------------------------------------------CARDS------------------------------------------------------------';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= 'QUANTIDADE TCs: Quantidade de vendas consolidads com o status de concluídas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TOTAL DE VENDAS(R$): Valor total de vendas consolidads com o status de concluídas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TICKET MÉDIO(R$): Valor de Ticket Médio de acordo vendas consolidads!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDAS CCANCELADAS (PÓS VENDA): Quantidade de vendas consolidads que foram canceladas pós vendas!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'FECHAMENTO DE CAIXA PENDENTES (30 DIAS): Quantidade de caixa pendentes consolidados dos últimos 30 dias de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'APROVAÇÃO DE MOVIMENTAÇÃO PENDENTES (30 DIAS): Quantidade de cupons pendentes consolidados dos últimos 30 dias de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'DIFERENÇA DE CAIXAS APURADAS (DIA ANTERIOR): Valor das diferenças dos caixas consolidados dos último dia de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TRANSMISSÕES DE CUPOM PENDENTES (30 DIAS): Quantidade de cupons pendentes de transmissão consolidados dos últimos 30 dias de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'BAIXA DE ITENS DE ESTOQUE PENDENTES (30 DIAS): Quantidade de itens pendentes consolidados para baixa de estoque dos últimos 30 dias de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'PRODUTOS SEM NCM OU ALÍQUOTA DE IMPOSTO: Quantidade de produtos sem parametrização de NCM ou Alíquotas consolidados de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'NF DE ENTRADA EM DIGITAÇÃO: Quantidade de notas fiscais de entrada em digitação de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'NF DE ENTRADA DIGITADAS: Quantidade de notas fiscais de entrada digitadas de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'NF DE ENTRADA IMPORTADAS(XML): Quantidade de notas fiscais importadas de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'NF DE SAÍDA EM DIGITAÇÃO: Quantidade de notas fiscais de saída em digitação de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'NF DE SAÍDA PENDENTES: Quantidade de notas fiscais de saída pendentes de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'NF DE SAÍDA DIGITADAS: Quantidade de notas fiscais de saída digitadas de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENCIMENTO: Dias para o vencimento do certificado digital! OBS.: Esse card aparece só quando está para vencer!';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= '-----------------------------------------------------GRAFICOS------------------------------------------------------------';
            $mensagem .= PHP_EOL;
            $mensagem .= PHP_EOL;
            
            $mensagem .= 'VENDAS MENSAL: Valores mensal do ano atual, anterior e da meta estipulada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'QUANTIDADE DE TICKETS - TCs: Quantidade de tickets mensal do ano atual e anterior!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TICKETS MÉDIO - TM: Valor do Ticket Médio mensal do ano atual e anterior!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TOTAL POR MODALIDADE ANUAL: Valor/Porcentagem por modalidade anual!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'VENDAS POR UNIDADE NO DIA: Valor/Porcentagem por Unidade de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
            $mensagem .= 'TIPO DE RECEBIMENTO NO DIA: Valor total por tipo de recebimento de acordo com a data filtrada!';
            $mensagem .= PHP_EOL;
        }
        
        $getInfo['informacoes'] = $mensagem;
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getInfo', $getInfo));   
    }
    
    
    public function findVisible(DTO\Request\Filter $request, DTO\Response $response){
        //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        $getVisible = [];
        $retorno = $this->arrayVisible($type);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVisible', $retorno));   
    }
    
    private function arrayVisible($type){
        
        $visible = [];
        
        $cards = $this->arrayCards();
        foreach($cards as $key => $value){
            
            if($type == 'P'){
                if($value['name'] == 'SALESCONCLUDED' || $value['name'] == 'SALESCANCELED' || $value['name'] == 'FECHCUPOMPEND' 
                || $value['name'] == 'VENDASNAOCONSOLIDADAS' || $value['name'] == 'FECHCAIXPEND' || 
                $value['name'] == 'CNCCONCERROSCARD'){
                       $cards[$key]['visible'] = true;
                }else{
                    $cards[$key]['visible'] = false;
                }    
            }else{
                if($value['name'] == 'SALESCONCLUDED' || $value['name'] == 'SALESCANCELED' || $value['name'] == 'FECHCUPOMPEND' 
                || $value['name'] == 'VENDASNAOCONSOLIDADAS' || $value['name'] == 'FECHCAIXPEND' || 
                $value['name'] == 'CNCCONCERROSCARD'){
                       $cards[$key]['visible'] = false;
                }else{
                    $cards[$key]['visible'] = true;
                }    
            }
        }
        
        $tables = array_merge($this->arrayTables(), $this->arrayTablesCusto());
        foreach($tables as $key => $value){
            if($type == 'C'){
                $tables[$key]['visible'] = true;
            }
        }
        
        $graficos = $this->arrayGraficos($type);
        $visible = array_merge($cards, $graficos, $tables);
        
        return $visible;
    }
    
    
    public function findCardsDashboard (DTO\Request\Filter $request, DTO\Response $response) {
		
         //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        
        $retornoCards = $this->arrayCards();
        
        $condition = $this->setConditions($request);
        $this->setTCDandCDcondition($condition, 'CDFILIAL', 'CDFILIAL');
        unset($condition['FILTERDASH']);
        
        //Cards de acordo com o tipo de Dashboard, busca conjunto de cards
        if($type == 'P'){
             $this->findCardsPadrao($retornoCards, $condition);
        }
        
        if($type == 'C'){
            $this->findCardsCorp($retornoCards, $condition);
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getCards', $retornoCards));   
    }
    
    public function findTablesDashboard (DTO\Request\Filter $request, DTO\Response $response) {
		
         //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        
        $retornoTables = $this->arrayTables();
        
        $condition = $this->setConditions($request);
        $this->setTCDandCDcondition($condition, 'CDFILIAL', 'CDFILIAL');
        unset($condition['FILTERDASH']);
        
        //busca dados das tabelas
        if($type == 'C'){
            $this->findTablesCorp($retornoTables, $condition);
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTables', $retornoTables));   
    }
    
    public function findTablesCustoDashboard (DTO\Request\Filter $request, DTO\Response $response) {
		
         //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        
        $retornoTables = $this->arrayTablesCusto();
        
        $condition = $this->setConditions($request);
        $this->setTCDandCDcondition($condition, 'CDFILIAL', 'CDFILIAL');
        unset($condition['FILTERDASH']);
        
        //busca dados das tabelas
        if($type == 'C'){
            $this->findTablesCustoCorp($retornoTables, $condition);
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTablesCusto', $retornoTables));   
    }
    
    private function findTablesCorp(&$retornoTables, $condition){
        
        //consultas menos demoradas
        $retornoProdMaisAjustes = $this->ManageDashboardService->findProdMaisAjustes($condition);
        $retornoContasPagar = $this->ManageDashboardService->findContasPagar($condition);
        $retornoContasReceber = $this->ManageDashboardService->findContasReceber($condition);
        $retornoSaldoEmContas = $this->ManageDashboardService->findSaldoEmContas($condition);
        $retornoStatusNota = $this->ManageDashboardService->findStatusNotaTab($condition);
        $retornoSemContagem = $condition['TCDFILIAL'] == 'T' ? $this->ManageDashboardService->findUnidadeSemContagem($condition) : $this->ManageDashboardService->findUltimaContagem($condition);
        
        $retornoCMVTab = $this->ManageDashboardService->findCMVTab($condition);
        
        foreach ($retornoTables as $key => $values) {
            $retornoTables[$key]['visible'] = true;
            if(($values['name'] == 'PRODMAISAJUST')){
                $retornoTables[$key]['value'] = $retornoProdMaisAjustes;
            }
            if(($values['name'] == 'CONTASPAGAR')){
                $retornoTables[$key]['value'] = $retornoContasPagar;
            }
            if(($values['name'] == 'CONTASRECEBER')){
                $retornoTables[$key]['value'] = $retornoContasReceber;
            }
            if(($values['name'] == 'SALDOEMCONTAS')){
                $retornoTables[$key]['value'] = $retornoSaldoEmContas;
            }
            if(($values['name'] == 'STATUSNOTA')){
                $retornoTables[$key]['value'] = $retornoStatusNota;
            }
            if(($values['name'] == 'CONTAGEMPEND')){
                $retornoTables[$key]['value'] = $retornoSemContagem;
            }
            if(($values['name'] == 'CMV')){
                $retornoTables[$key]['value'] = $retornoCMVTab;
            }
        }
        
    }
    
    private function findTablesCustoCorp(&$retornoTables, $condition){
        
        //consultas demoradas
        $retornoProdutoMaiorCusto = $this->ManageDashboardService->findProdutoMaiorCusto($condition);
        $retornoProdutoMenorCusto = $this->ManageDashboardService->findProdutoMenorCusto($condition);
        
        foreach ($retornoTables as $key => $values) {
            $retornoTables[$key]['visible'] = true;
            if(($values['name'] == 'MAIORCUSTO')){
                $retornoTables[$key]['value'] = $retornoProdutoMaiorCusto;
            }
            if(($values['name'] == 'MENORCUSTO')){
                $retornoTables[$key]['value'] = $retornoProdutoMenorCusto;
            }
        }
        
    }
    
    public function findTypeGraficosDashboard (DTO\Request\Filter $request, DTO\Response $response) {
        
        //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        $getInfos = [];
        
        //Graficos de acordo com o tipo de Dashboard, busca conjunto de cards
        $retornoGraficos = $this->arrayGraficos($type);
        
        $getInfos['type'] = $type;
        $getInfos['graficos'] = $retornoGraficos;
        
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getInfos', $getInfos));   
    }
    
    public function findGraficosDashboard (DTO\Request\Filter $request, DTO\Response $response) {
        
        //pega o tipo de Dashboard
        $type = $this->getTypeDashBoard();
        
        $retornoGraficos = $this->arrayGraficos($type);
        
        $condition = $this->setConditions($request);
        $this->setTCDandCDcondition($condition, 'CDFILIAL', 'CDFILIAL');
        $this->setTCDandCDcondition($condition, 'CDFILIALP', 'CDFILIALP');
        unset($condition['FILTERDASH']);
        
        //Values de acordo com o tipo de Dashboard, busca conjunto de cards
        if($type == 'P'){
             $this->findValuesGraficosPadrao($retornoGraficos, $condition);
        }
        
        if($type == 'C'){
            $this->findValuesGraficosCorp($retornoGraficos, $condition);
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getGraficos', $retornoGraficos));   
    }
    
    private function getTypeDashBoard(){
        
        $type = 'P';
		
        //busca typeDashboard
        $typeDashboard = $this->ManageDashboardService->findGetTypeDash();
        
        if(!empty($typeDashboard)){
            $type = $typeDashboard['IDDASHBOARD'];
        }
        
        return $type;
    }
    
    private function findValuesGraficosCorp(&$retornoGraficos, $condition){
        
        $retornoGraph = $this->ManageDashboardService->findSaleGraph($condition);
        $retornoSaleTypeGraph = $this->ManageDashboardService->findSaleTypeGraph($condition);
        $retornoSalePaymentGraph = $this->ManageDashboardService->findSalePaymentGraph($condition);
        $retornoSaleForDayGraph = $this->ManageDashboardService->findSaleForDayGraph($condition);
        
        foreach ($retornoGraficos as $key => $values) {
            if(($values['name'] == 'VENDASREALIZADAS') || ($values['name'] == 'GRAPHQTDTCS') || ($values['name'] == 'GRAPHTICKETMEDIO')){
                $retornoGraficos[$key]['value'] = $retornoGraph;
            }
            if(($values['name'] == 'GRAPHMODVEND')){
                $retornoGraficos[$key]['value'] = $retornoSaleTypeGraph;
            }
            if(($values['name'] == 'GRAPHPAYMENT')){
                $retornoGraficos[$key]['value'] = $retornoSalePaymentGraph;
            }
            if(($values['name'] == 'GRAPHSALESFORDAY')){
                $retornoGraficos[$key]['value'] = $retornoSaleForDayGraph;
            }
        }
        
    }
    
    private function findValuesGraficosPadrao(&$retornoGraficos, $condition){
        
        $retornoTotalRealPorFilial = $this->ManageDashboardService->findSaleGraphPorFilial($condition);
        $arrayVaziaVendaHora = true;
        
        if(!empty($retornoTotalRealPorFilial)){
            if(isset($retornoTotalRealPorFilial[0]['VAZIO'])){
                $arrayVaziaVendaHora = false;    
            }
        }
        
        $totalVenda = $this->ManageDashboardService->getTotalVenda($condition);
        $vendaPorHora = $this->ManageDashboardService->getVendaHora($condition, $arrayVaziaVendaHora);
        
        $ticketMedio = $this->ManageDashboardService->getTicketMedio($condition);
        $ticketMedioFilial = $this->ManageDashboardService->getTicketMedioFilial($condition);
        
        $vendaModalidade = $this->ManageDashboardService->getVendaModalidade($condition);
        $vendaModalidadeFilial = $this->ManageDashboardService->getVendaModalidadeFilial($condition);
        
        foreach ($retornoGraficos as $key => $values) {
            if(($values['name'] == 'VENDASREALIZADASP') || ($values['name'] == 'DONUTVENDAS')){
                $retornoGraficos[$key]['allData']['totalVendasPorFilial'] = $retornoTotalRealPorFilial;
                $retornoGraficos[$key]['allData']['totalVendas'] = $totalVenda;
                $retornoGraficos[$key]['allData']['vendaHora'] = $vendaPorHora;
            }
            
            if(($values['name'] == 'TICKETMEDIOP')){
                $retornoGraficos[$key]['allData']['ticketMedio'] = $ticketMedio;
                $retornoGraficos[$key]['allData']['ticketMedioFilial'] = $ticketMedioFilial;
            }
            
            if(($values['name'] == 'VENDAMODALIDADEP')){
                $retornoGraficos[$key]['allData']['vendaModalidade'] = $vendaModalidade;
                $retornoGraficos[$key]['allData']['vendaModalidadeFilial'] = $vendaModalidadeFilial;
            }
        }
    }
    
    private function findCardsCorp(&$retornoCards, $condition){
        $retorno = $this->ManageDashboardService->findCardsCorp($condition);
        
        foreach ($retornoCards as $key => $values) {
            if(Functions::arrayKeyExists($values['name'], $retorno)){
                
                //condition VENCIMENTO
                if(($values['name'] == 'VENCIMENTO')){
                    $retornoVencimento = $this->validVencimento($retorno[$values['name']]);
                    $retornoCards[$key]['visible'] = $retornoVencimento['visible'];
                    if(!empty($retornoVencimento['message']) && $retornoVencimento['typeMessage']){
                        $retornoCards[$key]['message'] = $retornoVencimento['message'];
                        $retornoCards[$key]['typeMessage'] = $retornoVencimento['typeMessage'];
                        $retornoCards[$key]['color']=(isset($retornoVencimento['color']))? $retornoVencimento['color'] :null;
                    }
                    $retorno[$values['name']]=(isset($retornoVencimento['value']))? $retornoVencimento['value'] :$retorno[$values['name']];
                }else{
                    $retornoCards[$key]['visible'] = true;
                }
                
                if(($values['name'] == 'QTTICKETCARD') || ($values['name'] == 'TOTVENDCARD') || ($values['name'] == 'AVGTICKETCARD')){
                    $retornoCards[$key]['color'] = $this->HandleColorsSales($retorno[$values['name']]);
                }else if(($values['name'] == 'DIFAPURACARD')){
                    $retornoCards[$key]['color'] = $this->HandleColorsNegative($retorno[$values['name']]);
                }else if(($values['name'] == 'CANCTICKET') || ($values['name'] == 'FECHACAIXAPEND') || ($values['name'] == 'APROVMOVCARD') ||
                    ($values['name'] == 'CUPOMPENDCARD') || ($values['name'] == 'ITESTPENDCARD') || ($values['name'] == 'PRODIRREG')
                ){
                    $retornoCards[$key]['color'] = $this->HandleColors($retorno[$values['name']]);
                }
                
                //formatValoresReais
                if(($values['name'] == 'TOTVENDCARD') || ($values['name'] == 'AVGTICKETCARD') || ($values['name'] == 'DIFAPURACARD')){
                    $retornoCards[$key]['value'] = number_format($retorno[$values['name']], 2, ',', '.');
                    
                    if($values['name'] == 'DIFAPURACARD'){
                        $retornoCards[$key]['value'] = 'Vr.:'.$retornoCards[$key]['value'];
                    }
                    
                }else{
                    $retornoCards[$key]['value'] = $retorno[$values['name']];
                }
            }
        }
    }
    
    private function findCardsPadrao(&$retornoCards, $condition){
        $retorno = $this->ManageDashboardService->findCardsPadrao($condition);
        
        foreach ($retornoCards as $key => $values) {
            if(Functions::arrayKeyExists($values['name'], $retorno)){
                
                //condition CNC visualizar 
                if(($values['name'] == 'CNCCONCERROSCARD') && $retorno[$values['name']] == 0){
                    $retornoCards[$key]['visible'] = false;    
                }else{
                    $retornoCards[$key]['visible'] = true;
                }
                $retornoCards[$key]['value'] = $retorno[$values['name']];
                
                if(!($values['name'] == 'SALESCONCLUDED') && !($values['name'] == 'SALESCANCELED')){
                    $retornoCards[$key]['color'] = $this->HandleColorsNegative($retorno[$values['name']]);    
                }
            }
        }
    }
    
    private function arrayCards(){
        $retornoCards = [];
        $arrayCards = ['QTTICKETCARD', 'TOTVENDCARD', 'CANCVENDCARD', 'FECHCAIXPENDCARD', 'APROVMOVCARD', 
        'DIFAPURACARD' , 'CUPOMPENDCARD', 'ITESTPENDCARD', 'AVGTICKETCARD', 'PRODIRREG', 
        'QTNFEDIGIT', 'QTNFEPEND', 'QTNFEDIGITHAND', 'QTNFEIMPORTXML', 'QTNFSDIGIT', 'QTNFSPEND',
        'QTNFSDIGITHAND', 'VENCIMENTO',
        'SALESCONCLUDED', 'SALESCANCELED', 'FECHCUPOMPEND', 'VENDASNAOCONSOLIDADAS', 'FECHCAIXPEND', 'CNCCONCERROSCARD'];
        
        foreach ($arrayCards as $key => $values) {
            $retornoCards[$key]['visible'] = false;
        	$retornoCards[$key]['name'] = $values;
        	$retornoCards[$key]['value'] = 0;  
        }
        return $retornoCards;
    }
    
    private function arrayTables(){
        $retornoTables = [];
        $arrayTables = ['PRODMAISAJUST', 'CONTAGEMPEND', 'CONTASPAGAR', 
        'CONTASRECEBER' , 'SALDOEMCONTAS', 'CMV', 'STATUSNOTA'];
        
        foreach ($arrayTables as $key => $values) {
            $retornoTables[$key]['visible'] = false;
        	$retornoTables[$key]['name'] = $values;
        }
        
        return $retornoTables;
    }
    
    private function arrayTablesCusto(){
        $retornoTables = [];
        $arrayTables = ['MAIORCUSTO', 'MENORCUSTO'];
        
        foreach ($arrayTables as $key => $values) {
            $retornoTables[$key]['visible'] = false;
        	$retornoTables[$key]['name'] = $values;
        }
        
        return $retornoTables;
    }
    
    private function arrayGraficos($type){
        $retornoGraficos = [];
        $arrayGraficosCorp = ['VENDASREALIZADAS', 'GRAPHQTDTCS', 'GRAPHTICKETMEDIO', 'GRAPHMODVEND', 'GRAPHSALESFORDAY', 'GRAPHPAYMENT'];
        $arrayGraficosPadrao = ['VENDASREALIZADASP', 'DONUTVENDAS', 'VENDAMODALIDADEP', 'TICKETMEDIOP'];
        $i = 0;
        foreach ($arrayGraficosCorp as $values) {
            if($type == 'C'){
                $retornoGraficos[$i]['visible'] = true;
            }else{
                $retornoGraficos[$i]['visible'] = false;    
            }
        	$retornoGraficos[$i]['name'] = $values;
        	$i++;
        }
        
        foreach ($arrayGraficosPadrao as $values) {
            if($type == 'P'){
                $retornoGraficos[$i]['visible'] = true;
            }else{
                $retornoGraficos[$i]['visible'] = false;    
            }
        	$retornoGraficos[$i]['name'] = $values;
        	$i++;
        }
        
        return $retornoGraficos;
    }
    
    private function HandleColorsSales ($value) {
        if ($value == 0){
            return '#b51313';
        }else{
            return '#18b28a';
        }
    }
    
    private function HandleColors ($value) {
        if ($value > 0){
            return '#b51313';
        }else{
            return '#18b28a';
        }
    }
    
    private function HandleColorsNegative($value) {
        if ($value == 0){
            return '#18b28a';
        }else{
            return '#b51313';
        }
    }
    
    private function validVencimento($value) {
        $cardVencimento = [];
        $cardVencimento['message'] = null;
        $cardVencimento['typeMessage'] = null;
        $cardVencimento['visible'] = false;
        if ($value == 'V') {
            $cardVencimento['visible'] = false;
        }
       else if ($value == 'SC') {
            $cardVencimento['message'] = 'Atenção! Unidade possui caixas NFC-e, e não possui certificado digital configurado.';
            $cardVencimento['typeMessage'] = 'E';
            $cardVencimento['visible'] = false;
        }
        else if ($value <= 5 && $value > 0) {
            $cardVencimento['message'] = 'Atenção! Certificado digital vence em ' .$value. ' dia.';
            $cardVencimento['typeMessage'] = 'W';
            $cardVencimento['visible'] = true;
            $cardVencimento['color']='#b51313';
            $cardVencimento['value']=$value;
        }
        else if ($value < 30 && $value > 0) {
            $cardVencimento['message'] = 'Atenção! Certificado digital vence em ' .$value .' dias';
            $cardVencimento['typeMessage'] = 'W';
            $cardVencimento['visible'] = true;
            $cardVencimento['value']=$value;
            $cardVencimento['color']='#18b28a';
        }
        else if ($value < 90 && $value > 0) {
            $cardVencimento['visible'] = true;
            $cardVencimento['value']=$value;
        }
         else if ($value < 0 || $value == 0 ) {
                $value = $value * (-1);
                //$today = moment().add(cards.VENCIMENTO, 'd').format('DD/MM/YYYY');
                $today = new \DateTime();
                $today->sub(new \DateInterval('P'.$value.'D'));
    
                $cardVencimento['message'] = 'Atenção! ' .$value. ' dias sem transmitir notas. Certificado Vencido desde: '.$today->format('d/m/Y');
                $cardVencimento['typeMessage'] = 'E';
                $cardVencimento['visible'] = true;
                $cardVencimento['color']='#b51313';
                $cardVencimento['value']='Vencido';
            }
        
        return $cardVencimento;
    }
    
    private function setTCDandCDcondition(&$condition, $key, $keyName) {
        if(!Functions::arrayKeyExists($key, $condition) || $condition[$keyName] == "" || empty($condition[$keyName])) {
            $condition['T'.$keyName] = 'T';    
            $condition[$keyName] = null;    
        } else {
            if(is_array($condition[$key])) {
                $condition[$keyName] = '_'.implode('_', $condition[$keyName] ).'_';
            }
            
            $condition['T'.$keyName] = NULL;
            
            if($key !== $keyName) {
                unset($condition[$keyName]);
            }
        }
    }
}